# -*- coding: utf8 -*-
from __future__ import unicode_literals

import base64
import logging

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    HprimDate,
    HprimForeign,
    HprimString,
    HprimList,
    HprimMultiple,
    Segment,
)
from driverng.message import BinGraphContainer

logger = logging.getLogger("driverng.aquios")


class AquiosGraphContainer(BinGraphContainer):
    def __init__(self, data=None):
        super(AquiosGraphContainer, self).__init__()
        self["data"] = data
        self["format"] = "jpeg"
        self["graph_type"] = "binary"


class AquiosSegmentH(HeaderSegment):
    sender_name = HprimString((5, 1))
    device_id = HprimString((5, 2))
    version = HprimString(13, default="LIS2-A2")
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)
    infos = HprimForeign("M", "header")


class AquiosSegmentP(Segment):
    laboratory_patient_id = HprimString(4)
    government_id = HprimString(5)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    middle_name = HprimString((6, 3))
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    sex = HprimString(9)
    physician_id = HprimString((14, 1))
    physician_last_name = HprimString((14, 2))
    physician_first_name = HprimString((14, 3))
    physician_middle_name = HprimString((14, 4))
    admission_status = HprimString(25)
    location = HprimString(26)

    orders = HprimForeign("O", "patient")


class AquiosSegmentO(Segment):
    specimen_id = HprimString(3)
    tests = HprimList(
        5,
        basefield=HprimMultiple(
            0,
            identifier=HprimString(1),
            name=HprimString(2),
            type=HprimString(3),
            code=HprimString(4),
        ),
    )
    priority = HprimString(6)
    collection_date = HprimDate(8)
    action_code = HprimString(12)
    specimen_type = HprimString(16)
    physician_id = HprimString((17, 1))
    physician_last_name = HprimString((17, 2))
    physician_first_name = HprimString((17, 3))
    physician_middle_name = HprimString((17, 4))
    record_type = HprimString(26, default="O")

    results = HprimForeign("R", "order")
    infos = HprimForeign("M", "order")


class AquiosSegmentR(Segment):
    test_id = HprimString((3, 4))
    replicate = HprimString((3, 5))
    reagent_lot = HprimString((3, 6))
    reagent_serial = HprimString((3, 7))
    value = HprimString((4, 1))
    interpretation = HprimString((4, 3))
    units = HprimString(5)
    reference_ranges = HprimString(6)
    abnormal_flags = HprimString(7)
    result_status = HprimString(9)
    complete_date = HprimDate(13)
    instrument = HprimString((14, 1))
    device_id = HprimString((14, 2))


class AquiosSegmentC(Segment):
    source = HprimString(3)
    text = HprimString(4)
    type = HprimString(5)


class AquiosSegmentQ(Segment):
    patient_id = HprimString((3, 1))
    specimen_id = HprimString((3, 2))
    request_status = HprimString(13)


class AquiosSegmentL(Segment):
    termination_code = HprimString(3, default="N")


class AquiosSegmentM(Segment):
    test_id = HprimString((3, 4))
    value = HprimString(4)
    result_status = HprimString(9)
    completed_date = HprimDate(13)
    instrument = HprimString((14, 1))
    device_id = HprimString((14, 2))


class Driver(AstmMessageV2):
    SPEC = {"code": "aquios", "label": "Beckman Aquios"}
    SEGMENT_CLASS_MAP = {
        "H": AquiosSegmentH,
        "P": AquiosSegmentP,
        "O": AquiosSegmentO,
        "R": AquiosSegmentR,
        "C": AquiosSegmentC,
        "Q": AquiosSegmentQ,
        "L": AquiosSegmentL,
        "M": AquiosSegmentM,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    OPTIONS = [
        AstmMessageV2.Option(
            "aquios_sender_name", label="Paramétrer le nom de l'émetteur du segment H"
        )
    ]

    def __init__(self, **kwargs):
        self.aquios_sender_name = kwargs.pop("aquios_sender_name", "MCA")
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = AquiosSegmentH(type_segment="H", **kwargs)
        message.end = AquiosSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        msg = self.create(date=context["now"], sender_name=self.aquios_sender_name)
        if not msg.sender_name:
            msg.sender_name = worklist.sender_id
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            action={"del": "C", "add": "A", "new": "A"},
            priority={"A": "S", "S": "S", worklist.ELSE: "R"},
        ):
            physician = [
                p for p in patient.physicians if p.type.lower() == "prescriber"
            ]
            seg_p = msg.patients.add(
                laboratory_patient_id=patient.patient_id,
                last_name=patient.last_name,
                first_name=patient.first_name,
                maiden_name=patient.maiden_name,
                birthdate=patient.birthdate,
                sex=patient.sex,
                location=patient.location,
                physician_id=physician[0].code if physician else "",
                physician_last_name=physician[0].last_name if physician else "",
                physician_first_name=physician[0].first_name if physician else "",
            )
            for sample, action, priority in patient.samples:
                seg_o = seg_p.orders.add(
                    specimen_id=sample.sample_id,
                    action_code=action,
                    priority=priority,
                    collection_date=sample.collection_date,
                    specimen_type=sample.specimen,
                )
                for order in sample.orders:
                    t = seg_o.tests.add()
                    t.code = order.code

        if msg.patients:
            return msg

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(type="patient", sample_id=seg_q.specimen_id)
            return worklist

        worklist = Worklist(context="result")
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                sample_id = seg_o.specimen_id
                comments = []
                if seg_o.commentaires:
                    for seg_c in seg_o.commentaires:
                        comments.append(seg_c.text)
                # Histograms
                if seg_o.infos:
                    sample = worklist.get_or_create(sample_id=sample_id, type="patient")
                    for seg_m in seg_o.infos:
                        if len(seg_m.value) % 4:
                            # if not a multiple of 4, add padding for base64 decoding
                            seg_m.value += "=" * (4 - len(seg_m.value) % 4)
                        sample.results.add(
                            code=seg_m.test_id,
                            value="GRAPHE",
                            date=seg_m.completed_date,
                            instrument="{}_{}".format(
                                seg_m.instrument, seg_m.device_id
                            ),
                            graph=AquiosGraphContainer(
                                data=base64.b64decode(seg_m.value)
                            ),
                        )
                for seg_r in seg_o.results:
                    # QC
                    if "*" in seg_r.test_id and "**" not in seg_r.test_id:
                        sample = worklist.get_or_create(
                            sample_id=sample_id, control_lot=sample_id, type="control"
                        )
                    # Patient
                    else:
                        sample = worklist.get_or_create(
                            sample_id=sample_id, type="patient"
                        )
                    unit = ""
                    flags = []
                    if seg_r.units:
                        unit = seg_r.units
                    if seg_r.abnormal_flags:
                        flags.append(seg_r.abnormal_flags)
                    result = sample.results.add(
                        code=seg_r.test_id,
                        value=seg_r.value,
                        date=seg_r.complete_date,
                        instrument="{}_{}".format(seg_r.instrument, seg_r.device_id),
                        unit=unit,
                        flags=flags,
                        comments=comments,
                    )
                    if seg_r.interpretation:
                        sample.results.add(
                            code="QUAL-{}".format(seg_r.test_id),
                            value=seg_r.interpretation,
                        )
                    result.lots.add(
                        type="reagent",
                        lot="{}_{}".format(seg_r.reagent_lot, seg_r.reagent_serial),
                    )
        return worklist
