# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import AstmMessageV2, HeaderSegment
from driverng.astm import HprimDate as HL7Date
from driverng.astm import HprimForeign as HL7Foreign
from driverng.astm import HprimString as HL7String
from driverng.astm import Segment


class BiosensorSegmentMSH(HeaderSegment):
    sending_application = HL7String((3, 1))
    sending_facility = HL7String(4)
    date = HL7Date(7)
    message_type = HL7String((9, 1), default="ORU")
    message_code = HL7String((9, 2), default="R01")
    message_label = HL7String((9, 3), default="ORU_R01")
    control_id = HL7String(10)
    processing_id = HL7String(11, default="P")
    version_id = HL7String(12)
    accept_type = HL7String(15, default="AL")
    application_type = HL7String(16, default="NA")
    character_set = HL7String(18, default="UNICODE UTF-8")
    principal_language_message = HL7String(19)
    message_profile_identifier = HL7String(21)

    patients = HL7Foreign("PID", "header")
    orders = HL7Foreign("OBR", "header")


class BiosensorSegmentPID(Segment):
    patient_identifier = HL7String(4)

    orders = HL7Foreign("OBR", "header")


class BiosensorSegmentOBR(Segment):
    placer_order_number = HL7String((3, 1))
    filler_order_numer = HL7String((4, 1))
    universal_service_id_code = HL7String((5, 1))
    universal_service_id_label = HL7String((5, 2))
    observation_date = HL7Date(8)
    observation_end_date = HL7Date(9)

    results = HL7Foreign("OBX", "orders")
    infos = HL7Foreign("NTE", "result")


class BiosensorSegmentOBX(Segment):
    value_type = HL7String(3)
    observation_identifier_code = HL7String((4, 1))
    observation_identifier_label = HL7String((4, 2))
    observation_sub_id = HL7String(5)
    value = HL7String((6, 1))
    value_cwe = HL7String((6, 2))
    units = HL7String((7, 1))
    units_text = HL7String((7, 2))
    references_range = HL7String(8)
    abnormal_flags = HL7String(9)
    observation_result_status = HL7String(12)
    observation_date = HL7Date(15)
    observer = HL7String(17)
    analysis_date = HL7Date(20)

    infos = HL7Foreign("NTE", "result")


class BiosensorSegmentNTE(Segment):
    comment = HL7String(4)


class Driver(AstmMessageV2):
    SPEC = {"code": "biosensor", "label": "SD Biosensor HL7 Orgentec"}
    SEGMENT_CLASS_MAP = {
        "MSH": BiosensorSegmentMSH,
        "PID": BiosensorSegmentPID,
        "OBR": BiosensorSegmentOBR,
        "OBX": BiosensorSegmentOBX,
        "NTE": BiosensorSegmentNTE,
    }
    DEFAULT_SEPARATORS = "|^~\\&"
    FIRST_SEP_INDEX = 3
    SEGMENT_ENDING = "\r"

    def create(self, **kwargs):
        return BiosensorSegmentMSH(type_segment="MSH", **kwargs)

    def to_string(self, message, separators=None, **kwargs):
        return super(Driver, self).to_string(message, separators, **kwargs) + "\r"

    @staticmethod
    def get_lot(infos):
        for seg_nte in infos:
            left_index = seg_nte.comment.find("LotNo=") + 6
            right_index = seg_nte.comment.find(",SerialNo")
            return seg_nte.comment[left_index:right_index]

    @staticmethod
    def get_cut_off_index(infos):
        for seg_nte in infos:
            right_index = seg_nte.comment.find(",Value=")
            left_index = seg_nte.comment.find("Value=") + 6
            return [seg_nte.comment[:right_index], seg_nte.comment[left_index:]]

    def decode_message(self, message):
        worklist = Worklist(context="result")
        if message.patients:
            for seg_pid in message.patients:
                for seg_obr in seg_pid.orders:
                    sample = worklist.get_or_create(
                        sample_id=seg_pid.patient_identifier, type="patient"
                    )
                    if seg_obr.infos:
                        lot = control_lot = self.get_lot(seg_obr.infos)
                    for seg_obx in seg_obr.results:
                        if seg_obx.value_type == "CWE":
                            result = sample.results.add(
                                code="{} {}".format(
                                    seg_obx.observation_identifier_label, seg_obx.value
                                ),
                                date=seg_obx.analysis_date,
                                operator=seg_obx.observer,
                                value=seg_obx.value_cwe,
                                flags=[seg_obx.observation_result_status],
                            )
                        elif seg_obx.value_type == "NM":
                            result = sample.results.add(
                                code=seg_obx.observation_identifier_label,
                                date=seg_obx.analysis_date,
                                operator=seg_obx.observer,
                                value=seg_obx.value,
                                unit=seg_obx.units,
                                flags=[seg_obx.observation_result_status],
                            )
                        elif seg_obx.value_type == "" and seg_obx.value == "":
                            result = sample.results.add(
                                code="{} {}".format(
                                    seg_obx.observation_identifier_label,
                                    self.get_cut_off_index(seg_obx.infos)[0],
                                ),
                                date=seg_obx.analysis_date,
                                operator=seg_obx.observer,
                                value=self.get_cut_off_index(seg_obx.infos)[1],
                                flags=[
                                    seg_obx.abnormal_flags,
                                    seg_obx.observation_result_status,
                                ],
                            )
                        else:
                            result = sample.results.add(
                                code=seg_obx.observation_identifier_label,
                                date=seg_obx.analysis_date,
                                operator=seg_obx.observer,
                                value=seg_obx.value,
                                flags=[
                                    seg_obx.abnormal_flags,
                                    seg_obx.observation_result_status,
                                ],
                            )
                        if lot:
                            result.lots.add(type="reagent", lot=lot)
        else:
            for seg_obr in message.orders:
                control_lot = self.get_lot(seg_obr.infos)
                for seg_obx in seg_obr.results:
                    if seg_obx.observation_identifier_code.startswith("QC"):
                        sample = worklist.get_or_create(
                            sample_id=seg_obx.observation_identifier_code,
                            control_lot=control_lot,
                            type="control",
                        )
                        result = sample.results.add(
                            code=seg_obx.observation_identifier_label,
                            date=seg_obx.analysis_date,
                            operator=seg_obx.observer,
                            value=seg_obx.value,
                            flags=[
                                seg_obx.abnormal_flags,
                                seg_obx.observation_result_status,
                            ],
                        )
                        if seg_obx.value_type == "NM":
                            result.unit = seg_obx.units
        return worklist
